==> oemof_thermal-0.0.7.tar.gz: oemof_thermal-0.0.7/examples/compression_heatpump_and_chiller/airsource_heatpump_const_max_output.py <==
"""
Example on how to use the 'calc_cops' function to get the
COPs of an exemplary air-source heat pump (ASHP) and use the
pre-calculated COPs in a solph.Converter.

We use the ambient air as low temperature heat reservoir.
"""

import os

import matplotlib.pyplot as plt
import numpy as np
import oemof.solph as solph
import pandas as pd

import oemof.thermal.compression_heatpumps_and_chillers as cmpr_hp_chiller


def airource_hp_const_example():
    # Set paths
    data_path = os.path.join(
        os.path.dirname(__file__), "data/ASHP_example.csv"
    )

    # Read input data
    data = pd.read_csv(data_path)

    # Set up an energy system model
    solver = "cbc"
    number_of_time_steps = 24
    solver_verbose = False

    date_time_index = pd.date_range(
        "1/1/2012", periods=number_of_time_steps, freq="H"
    )

    energysystem = solph.EnergySystem(timeindex=date_time_index)

    b_el = solph.Bus(label="electricity")

    b_heat = solph.Bus(label="heat")

    energysystem.add(b_el, b_heat)

    energysystem.add(
        solph.components.Source(
            label="grid_el", outputs={b_el: solph.Flow(variable_costs=10)}
        )
    )

    energysystem.add(
        solph.components.Source(
            label="backup_heating",
            outputs={b_heat: solph.Flow(variable_costs=10)},
        )
    )

    energysystem.add(
        solph.components.Sink(
            label="demand",
            inputs={
                b_heat: solph.Flow(fix=data["demand_heat"], nominal_value=1)
            },
        )
    )

    temp_threshold_icing = 2

    # Precalculation of COPs
    cops_ASHP = cmpr_hp_chiller.calc_cops(
        temp_high=[40],
        temp_low=data["ambient_temperature"],
        quality_grade=0.4,
        mode="heat_pump",
        temp_threshold_icing=temp_threshold_icing,
        factor_icing=0.8,
    )

    # Air-Source Heat Pump
    energysystem.add(
        solph.components.Converter(
            label="ASHP",
            inputs={b_el: solph.Flow()},
            outputs={b_heat: solph.Flow(nominal_value=25, variable_costs=5)},
            conversion_factors={b_heat: cops_ASHP},
        )
    )

    # Create and solve the optimization model
    model = solph.Model(energysystem)
    model.solve(solver=solver, solve_kwargs={"tee": solver_verbose})

    # Get results
    energysystem.results["main"] = solph.processing.results(model)
    energysystem.results["meta"] = solph.processing.meta_results(model)

    energysystem.dump(dpath=None, filename=None)

    # Processing the results
    energysystem = solph.EnergySystem()
    energysystem.restore(dpath=None, filename=None)

    string_results = solph.views.convert_keys_to_strings(
        energysystem.results["main"]
    )
    ASHP_output = string_results["ASHP", "heat"]["sequences"].values
    demand_h = string_results["heat", "demand"]["sequences"].values
    ASHP_input = string_results["electricity", "ASHP"]["sequences"].values

    # Example plot
    _, axs = plt.subplots(3, 1, figsize=(8, 5), sharex=True)
    axs[0].plot(ASHP_output, label="heat output")
    axs[0].plot(demand_h, linestyle="--", label="heat demand")
    axs[1].plot(cops_ASHP, linestyle="-.")
    axs[2].plot(
        [-1, number_of_time_steps],
        [temp_threshold_icing, temp_threshold_icing],
        linestyle="--",
        color="red",
        label="threshold temperature",
    )
    axs[2].text(
        x=number_of_time_steps - 1,
        y=temp_threshold_icing,
        s="threshold temperature",
        ha="right",
        va="center",
        color="red",
        fontsize=10,
        bbox=dict(facecolor="white", edgecolor="white", alpha=1.0),
    )
    axs[2].plot(data["ambient_temperature"], label="ambient temperature")
    axs[0].set_title("Heat Output and Demand")
    axs[1].set_title("Coefficient of Performance")
    axs[2].set_title("Source Temperature (Ambient)")
    axs[0].legend()

    axs[0].grid()
    axs[1].grid()
    axs[2].grid()
    axs[0].set_xlim(0, number_of_time_steps)
    axs[0].set_ylabel("Heat flow in kW")
    axs[1].set_ylabel("COP")
    axs[2].set_ylabel("Temperature in $°$C")
    axs[2].set_xlabel("Time in h")
    plt.tight_layout()
    plt.show()

    print("********* Main results *********")
    print(
        "Total electricity consumption: {:2.1f}".format(
            ASHP_input.sum(axis=0)[0]
        )
    )
    print("Total heat output: {:2.1f}".format(ASHP_output.sum(axis=0)[0]))
    print(
        "Average Coefficient of Performance (COP): {:2.2f}".format(
            np.mean(cops_ASHP)
        )
    )
    print(
        "Seasonal Performance Factor (SPF): {:2.2f}".format(
            ASHP_output.sum(axis=0)[0] / ASHP_input.sum(axis=0)[0]
        )
    )


if __name__ == "__main__":
    airource_hp_const_example()

==> oemof_thermal-0.0.8.tar.gz: oemof_thermal-0.0.8/examples/compression_heatpump_and_chiller/airsource_heatpump_const_max_output.py <==
"""
Example on how to use the 'calc_cops' function to get the
COPs of an exemplary air-source heat pump (ASHP) and use the
pre-calculated COPs in a solph.Converter.

We use the ambient air as low temperature heat reservoir.
"""

import os

import matplotlib.pyplot as plt
import numpy as np
import oemof.solph as solph
import pandas as pd

import oemof.thermal.compression_heatpumps_and_chillers as cmpr_hp_chiller


def airource_hp_const_example():
    # Set paths
    data_path = os.path.join(
        os.path.dirname(__file__), "data/ASHP_example.csv"
    )

    # Read input data
    data = pd.read_csv(data_path)

    # Set up an energy system model
    solver = "cbc"
    number_of_time_steps = 24
    solver_verbose = False

    date_time_index = pd.date_range(
        "1/1/2012", periods=number_of_time_steps, freq="h"
    )

    energysystem = solph.EnergySystem(timeindex=date_time_index)

    b_el = solph.Bus(label="electricity")

    b_heat = solph.Bus(label="heat")

    energysystem.add(b_el, b_heat)

    energysystem.add(
        solph.components.Source(
            label="grid_el", outputs={b_el: solph.Flow(variable_costs=10)}
        )
    )

    energysystem.add(
        solph.components.Source(
            label="backup_heating",
            outputs={b_heat: solph.Flow(variable_costs=10)},
        )
    )

    energysystem.add(
        solph.components.Sink(
            label="demand",
            inputs={
                b_heat: solph.Flow(fix=data["demand_heat"], nominal_value=1)
            },
        )
    )

    temp_threshold_icing = 2

    # Precalculation of COPs
    cops_ASHP = cmpr_hp_chiller.calc_cops(
        temp_high=[40],
        temp_low=data["ambient_temperature"],
        quality_grade=0.4,
        mode="heat_pump",
        temp_threshold_icing=temp_threshold_icing,
        factor_icing=0.8,
    )

    # Air-Source Heat Pump
    energysystem.add(
        solph.components.Converter(
            label="ASHP",
            inputs={b_el: solph.Flow()},
            outputs={b_heat: solph.Flow(nominal_value=25, variable_costs=5)},
            conversion_factors={b_heat: cops_ASHP},
        )
    )

    # Create and solve the optimization model
    model = solph.Model(energysystem)
    model.solve(solver=solver, solve_kwargs={"tee": solver_verbose})

    # Get results
    energysystem.results["main"] = solph.processing.results(model)
    energysystem.results["meta"] = solph.processing.meta_results(model)

    energysystem.dump(dpath=None, filename=None)

    # Processing the results
    energysystem = solph.EnergySystem()
    energysystem.restore(dpath=None, filename=None)

    string_results = solph.views.convert_keys_to_strings(
        energysystem.results["main"]
    )
    ASHP_output = string_results["ASHP", "heat"]["sequences"].values
    demand_h = string_results["heat", "demand"]["sequences"].values
    ASHP_input = string_results["electricity", "ASHP"]["sequences"].values

    # Example plot
    _, axs = plt.subplots(3, 1, figsize=(8, 5), sharex=True)
    axs[0].plot(ASHP_output, label="heat output")
    axs[0].plot(demand_h, linestyle="--", label="heat demand")
    axs[1].plot(cops_ASHP, linestyle="-.")
    axs[2].plot(
        [-1, number_of_time_steps],
        [temp_threshold_icing, temp_threshold_icing],
        linestyle="--",
        color="red",
        label="threshold temperature",
    )
    axs[2].text(
        x=number_of_time_steps - 1,
        y=temp_threshold_icing,
        s="threshold temperature",
        ha="right",
        va="center",
        color="red",
        fontsize=10,
        bbox=dict(facecolor="white", edgecolor="white", alpha=1.0),
    )
    axs[2].plot(data["ambient_temperature"], label="ambient temperature")
    axs[0].set_title("Heat Output and Demand")
    axs[1].set_title("Coefficient of Performance")
    axs[2].set_title("Source Temperature (Ambient)")
    axs[0].legend()

    axs[0].grid()
    axs[1].grid()
    axs[2].grid()
    axs[0].set_xlim(0, number_of_time_steps)
    axs[0].set_ylabel("Heat flow in kW")
    axs[1].set_ylabel("COP")
    axs[2].set_ylabel("Temperature in $°$C")
    axs[2].set_xlabel("Time in h")
    plt.tight_layout()
    plt.show()

    print("********* Main results *********")
    print(
        "Total electricity consumption: {:2.1f}".format(
            ASHP_input.sum(axis=0)[0]
        )
    )
    print("Total heat output: {:2.1f}".format(ASHP_output.sum(axis=0)[0]))
    print(
        "Average Coefficient of Performance (COP): {:2.2f}".format(
            np.mean(cops_ASHP)
        )
    )
    print(
        "Seasonal Performance Factor (SPF): {:2.2f}".format(
            ASHP_output.sum(axis=0)[0] / ASHP_input.sum(axis=0)[0]
        )
    )


if __name__ == "__main__":
    airource_hp_const_example()
